//+------------------------------------------------------------------+
//|                                             3LineBreakSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "3LineBreak.ex5"
#include <Expert\ExpertSignal.mqh>
//--- wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    3LineBreak               |
//| Type=SignalAdvanced                                              |
//| Name=3LineBreak                                                  |
//| Class=C3LineBreakSignal                                          |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=period,int,3,                          |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
//--- wizard description end
//+------------------------------------------------------------------+
//|  C3LineBreakSignal.                                         |
//| :            |
//|  3LineBreak http://www.mql5.com/ru/code/485/.          |
//|                 CExpertSignal.        |
//+------------------------------------------------------------------+
class C3LineBreakSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      3LineBreak

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   3LineBreak
   uint              m_period;           //  
   uint              m_SignalBar;        //      

public:
                     C3LineBreakSignal();

   //---    
   void              BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void              SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void              BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void              SellPosClose(bool value)                { m_SellPosClose=value;     }
   void              Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void              period(uint value)                      { m_period=value;           }
   void              SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators);  //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              Init3LineBreak(CIndicators *indicators);  // ,   3LineBreak

  };
//+------------------------------------------------------------------+
//|  C3LineBreakSignal.                                   |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void C3LineBreakSignal::C3LineBreakSignal()
  {
//---    
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---   
   m_Ind_Timeframe=PERIOD_H4;
   m_period=3;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true -   ,  false.           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool C3LineBreakSignal::ValidationSettings()
  {
//---  
   if(m_period<=0)
     {
      printf(__FUNCTION__+":      ");
      return(false);
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool C3LineBreakSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---   
   if(!Init3LineBreak(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   3LineBreak.                             |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool C3LineBreakSignal::Init3LineBreak(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[2];
   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="3LineBreak.ex5";
   parameters[1].type=TYPE_INT;
   parameters[1].integer_value=m_period;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,2,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }
     
//---  
   if(!m_indicator.NumBuffers(3)) return(false);
   
//---     3LineBreak
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int C3LineBreakSignal::LongCondition()
  {
//---       2  3LineBreak
   double Signal0=m_indicator.GetData(2,m_SignalBar);
   double Signal1=m_indicator.GetData(2,m_SignalBar+1);

//---    
   if(Signal0==0)
     {
      if(Signal1==1 && m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }

      if(m_SellPosClose) return(CLOSE_SHORT);
     }
//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                           |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int C3LineBreakSignal::ShortCondition()
  {
//---       2  3LineBreak
   double Signal0=m_indicator.GetData(2,m_SignalBar);
   double Signal1=m_indicator.GetData(2,m_SignalBar+1);

//---    
   if(Signal0==1)
     {
      if(Signal1==0 && m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }

      if(m_BuyPosClose) return(CLOSE_LONG);
     }
//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+